@@ -41,7 +41,7 @@ class GuppyWindow:
 	# Quit command to put on transfer queue
 	QUIT_CMD = 'Quit'
 	
-	def __init__(self, datadir=''):	
+	def __init__(self, datadir='', dirname=None):	
 		# The PathBar widget only works with PyGtk 2.8 and greater
 		major, minor, micro = gtk.pygtk_version
 		if major <= 2 and minor < 8:
@@ -53,11 +53,12 @@ class GuppyWindow:
 		gtk.gdk.threads_init()
 
 		self.datadir = datadir
-
+		self.dirname = dirname
+		
 		# Find out proper way to find glade files
 		self.glade_file = self.datadir + '/' + 'guppy.glade'
 
-		self.pvr_error_window = PVRErrorWindow(self.glade_file)
+		self.pvr_error_window = PVRErrorWindow(self.glade_file, self.dirname)
 
 		actions = self.initUIManager()
 
@@ -65,7 +66,9 @@ class GuppyWindow:
 		gtk.glade.set_custom_handler(self.customWidgetHandler)
 		
 		# Load glade file
-		self.glade_xml = gtk.glade.XML(self.glade_file, None, gettext.textdomain())
+		self.glade_xml = gtk.glade.XML(self.glade_file,
+		                               None,
+		                               self.dirname)
 		
 		# Connect callback functions in glade file to functions
 		self.glade_xml.signal_autoconnect(self)
@@ -250,6 +253,16 @@ class GuppyWindow:
 		handler = getattr(self, func_name)
 		return handler(str1, str2, int1, int2)
 		
+	def changeDir(self, fs_model, dir=None):
+		try:
+			return fs_model.changeDir(dir)
+		except OSError:
+			if dir == None:
+				dir = fs_model.getCWD()
+			self.showErrorDialog(_('The folder contents could not be displayed.'),
+								 _('You do not have the permissions necessary to view the contents of "%s".') % dir)
+			return False
+		
 	def createFileTrees(self):	
 		self.pvr_treeview = self.glade_xml.get_widget('pvr_treeview')	
 		pvr_liststore = self.pvr_model.filter_new()
@@ -359,18 +372,20 @@ class GuppyWindow:
 		"""
 		
 		if warn:
-			msg = '<b>' + _('Are you sure you want to permanently delete') + '\n'
+			msg = _('Are you sure you want to permanently delete')
 			
 			files_len =  len(files)
 			if files_len > 1:
 				msg += ' ' + _('the %d selected items?') % files_len
 			else:
-				msg += ' "' + files[0] + '"?'
+				msg += ' "%s"?' % files[0]
 				
-			msg += '</b>\n\n' + _('If you delete an item, it is permanently lost.')
+			msg2 = _('If you delete an item, it is permanently lost.')
 			
-			dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_CANCEL)
-			dialog.set_markup(msg)
+			dialog = gtk.MessageDialog(type=gtk.MESSAGE_WARNING,
+			                           buttons=gtk.BUTTONS_CANCEL,
+			                           message_format=msg)
+			dialog.format_secondary_text(msg2)
 			dialog.add_button(gtk.STOCK_DELETE, 0)
 			
 			response = dialog.run()
@@ -380,28 +395,39 @@ class GuppyWindow:
 				return False
 		
 		retval = True
+		cancel = False
 		for name in files:
+			if cancel:
+				break
 			deleted = False
 			while not deleted:
 				try:
 					fs_model.delete(name)
 					deleted = True
 				except OSError:
-					SKIP, RETRY = range(2)
-					msg = '<b>' + _('Error while deleting.') + '</b>\n\n' + _('Cannot delete')
-					msg += ' "' + fs_model.getCWD() + '/' + name + '" '
-					msg += _('because you do not have permissions to change it or its parent folder.')
-					dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR)
-					dialog.set_markup(msg)
-					dialog.add_buttons( _('_Skip'), SKIP, _('_Retry'), RETRY)
+					dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
+					                           message_format=_('Error while deleting.'))
+					dialog.format_secondary_text(_('Cannot delete "%s" because you do not have permissions to change it or its parent folder.') % (fs_model.getCWD() + '/' + name))
+
+					if len(files) > 1:
+						dialog.add_button( _('_Skip'), gtk.RESPONSE_REJECT)
+					dialog.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
+					dialog.add_button(_('_Retry'), gtk.RESPONSE_ACCEPT)
+					dialog.set_default_response(gtk.RESPONSE_ACCEPT)
 					response = dialog.run()
 					dialog.destroy()
 		
 					# Skip this file and go to next file
-					if response == SKIP or response == gtk.RESPONSE_DELETE_EVENT:
+					if response == gtk.RESPONSE_REJECT:
 						retval = False
 						break
-				
+					
+					if response == gtk.RESPONSE_CANCEL or \
+					   response == gtk.RESPONSE_DELETE_EVENT:
+						retval = False
+						cancel = True
+						break
+
 					# Try to delete again
 					continue
 				except PuppyBusyError:
@@ -513,10 +539,10 @@ class GuppyWindow:
 		try:
 			name = fs_model.mkdir()
 		except OSError:
-			msg = '<b>' + _('Error while creating folder.') + '</b>\n\n'
-			msg += _('Cannot create folder because you do not have permission to write in its parent folder.')
-			dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE)
-			dialog.set_markup(msg)
+			dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
+			                           buttons=gtk.BUTTONS_CLOSE,
+			                           message_format=_('Error while creating folder.'))
+			dialog.format_secondary_text(_('Cannot create folder because you do not have permission to write in its parent folder.'))
 			response = dialog.run()
 			dialog.destroy()
 
@@ -559,40 +585,36 @@ class GuppyWindow:
 				fs_model.rename(old_name, new_name)
 				renamed = True
 			except OSError, error:
-				print 'error = ', error
-
 				# Handle Permission Denied error				
 				if str(error).find('Errno 13') != -1:
-					SKIP, RETRY = range(2)
-					msg = '<b>' + _('Error while renaming.') + '</b>\n\n' + _('Cannot rename')
-					msg += ' "' + fs_model.getCWD() + '/' + old_name + '" '
-					msg += _('because you do not have permissions to change it or its parent folder.')
-					dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR)
-					dialog.set_markup(msg)
-					dialog.add_buttons( _('_Skip'), SKIP, _('_Retry'), RETRY)
+					dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
+					                           message_format=_('Error while renaming.'))
+					dialog.format_secondary_text(_('Cannot rename "%s" because you do not have permissions to change it or its parent folder.') % (fs_model.getCWD() + '/' + old_name))
+					dialog.add_buttons(_('_Skip'), gtk.RESPONSE_REJECT,
+					                   _('_Retry'), gtk.RESPONSE_ACCEPT)
 					response = dialog.run()
 					dialog.destroy()
 		
 					# Skip this file and go to next file
-					if response == SKIP or response == gtk.RESPONSE_DELETE_EVENT:
+					if response == gtk.RESPONSE_REJECT or \
+					   response == gtk.RESPONSE_DELETE_EVENT:
 						retval = False
 						break
 				
 					# Try to rename again
 					continue
 				elif str(error).find('Errno 17') != -1:
-					CANCEL, REPLACE = range(2)
-					msg = '<b>' + _('The item could not be renamed') + '.</b>\n\n'
-					msg += _('The name') + ' "' + new_name + '" '
-					msg += _('is already used in this folder.')
-					dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR)
-					dialog.set_markup(msg)
-					dialog.add_buttons( _('Cancel'), CANCEL,_('_Replace'), REPLACE)
+					dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
+					                           message_format=_('The item could not be renamed.'))
+					dialog.format_secondary_text(_('The name "%s" is already used in this folder.') % new_name)
+					dialog.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+					                   _('_Replace'), gtk.RESPONSE_APPLY)
 					response = dialog.run()
 					dialog.destroy()
 		
 					# Skip this file and go to next path in files
-					if response == CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
+					if response == gtk.RESPONSE_CANCEL or \
+					   response == gtk.RESPONSE_DELETE_EVENT:
 						break
 		
 					deleted = self.deleteFiles([new_name], fs_model, False)
@@ -613,7 +635,7 @@ class GuppyWindow:
 	def on_path_entry_activate(self, widget, fs_model):
 		path = widget.get_text()
 
-		dir_changed = fs_model.changeDir(path)
+		dir_changed = self.changeDir(fs_model, path)
 	
 		if dir_changed == False:
 			widget.set_text(fs_model.getCWD())
@@ -753,8 +775,9 @@ class GuppyWindow:
 		"""
 		model, files = widget.get_selected_rows()
 		
-		file_count = len(files)
 		total_size = 0
+		# Don't enable action group unless are files selected.
+		enable_actiongrp = False
 		for path in files:
 			iter = model.get_iter(path)
 			type = model.get_value(iter, FileSystemModel.TYPE_COL)
@@ -762,11 +785,9 @@ class GuppyWindow:
 			size = model.get_value(iter, FileSystemModel.SIZE_COL)
 			if type != 'd':
 				total_size += convertToBytes(size)
+				enable_actiongrp = True
 
-		if total_size > 0:
-			msg = _('Selection Size') + ': ' + humanReadableSize(total_size)
-		else:
-			msg = _('Selection Size') + ':'
+		msg = _('Selection Size') + ': ' + humanReadableSize(total_size)
 		
 		if isinstance(fs_model, PCFileSystemModel):
 			label = self.pc_total_size_label
@@ -776,7 +797,7 @@ class GuppyWindow:
 			actiongrp = self.download_actiongrp
 			
 		label.set_text(msg)
-		if total_size > 0:
+		if enable_actiongrp > 0:
 			actiongrp.set_sensitive(True)
 		else:
 			actiongrp.set_sensitive(False)
@@ -800,7 +821,7 @@ class GuppyWindow:
 		type = model.get_value(iter, FileSystemModel.TYPE_COL)
 		
 		if type == 'd':
-			fs_model.changeDir(name)
+			self.changeDir(fs_model, name)
 			path = fs_model.getCWD()
 			if fs_model == self.pc_model:
 				if self.no_path_bar_support == False:
@@ -881,9 +902,14 @@ class GuppyWindow:
 		gtk.main_quit()
 
 	def showNoFileOperationDialog(self):
-		dialog = gtk.MessageDialog(message_format=_('It is not possible to delete, rename, or create folders on the PVR during a file transfer.'),
+		self.showErrorDialog(_('It is not possible to delete, rename, or create folders on the PVR during a file transfer.'))
+
+	def showErrorDialog(self, msg, msg2=None):
+		dialog = gtk.MessageDialog(message_format=msg,
 		                           type=gtk.MESSAGE_ERROR,
 		                           buttons=gtk.BUTTONS_CLOSE)
+		if msg2:
+			dialog.format_secondary_text(msg2)
 		dialog.run()
 		dialog.destroy()
 		
@@ -948,8 +974,13 @@ class GuppyWindow:
 		self.show_file_transfer_action.set_active(True)
 
 		existing_files = []
+		file_transfer = None
 		for path in files:
 			iter = model.get_iter(path)
+			
+			if model.get_value(iter, FileSystemModel.TYPE_COL) == 'd':
+				continue
+			
 			file = model.get_value(iter, FileSystemModel.NAME_COL)
 			date = model.get_value(iter, FileSystemModel.DATE_COL)
 
@@ -966,7 +997,9 @@ class GuppyWindow:
 				dst_file = dst_dir + '\\' + file
 				file_exists = self.pvr_model.find(file)
 
-			xml = gtk.glade.XML(self.glade_file, 'progress_box')
+			xml = gtk.glade.XML(self.glade_file,
+			                    'progress_box',
+			                    self.dirname)			                    
 			xml.signal_autoconnect(self)		
 
 			progress_box = xml.get_widget('progress_box')
@@ -1008,7 +1041,8 @@ class GuppyWindow:
 				msg2 = _('If you replace an existing file, its contents will be overwritten.')
 				dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION,
 				                           message_format=msg)
-				                           
+				dialog.format_secondary_text(msg2)
+				
 				if len(existing_files) > 1:
 					dialog.add_button(_('Replace _All'), REPLACE_ALL)
 				dialog.add_buttons( _('_Skip'), SKIP, _('_Replace'), REPLACE)
@@ -1112,7 +1146,7 @@ class GuppyWindow:
 					self.pvr_error_window.addError(_('Failed to get list of files'), error)
 					pass
 
-			model.changeDir()
+			self.changeDir(model)
 			
 			# Reselect rows
 			for path in selected_rows[1]:
@@ -1132,8 +1166,10 @@ class PVRErrorWindow:
 	LIST_TYPES.insert(MSG_COL, gobject.TYPE_STRING)
 	LIST_TYPES.insert(OUTPUT_COL, gobject.TYPE_STRING)
 
-	def __init__(self, glade_file):
-		glade_xml = gtk.glade.XML(glade_file, 'pvr_error_window')
+	def __init__(self, glade_file, dirname):
+		glade_xml = gtk.glade.XML(glade_file,
+		                          'pvr_error_window',
+		                          dirname)
 		# Connect callback functions in glade file to functions
 		glade_xml.signal_autoconnect(self)
 
@@ -1331,17 +1367,20 @@ class TransferThread(threading.Thread):
 			if self.guppy.turbo == True:
 				self.guppy.puppy.setTurbo(False)
 
-			# Set modification time of downloaded file to the same time as the
+			# Set modification time of downloaded file to the same time
 			# as on the PVR.
 			if transfer_successful and direction == 'download':
-				# Parse date string
-				time_struct = time.strptime(file_transfer.file_date, '%a %b %d %Y')
-				
-				# Convert to seconds since the epoch
-				time_secs = time.mktime(time_struct)
-				
-				# Set modification time
-				os.utime(dst_file, (int(time.time()), time_secs))
+				try:
+					# Parse date string
+					time_struct = time.strptime(file_transfer.file_date, '%a %b %d %Y')
+					
+					# Convert to seconds since the epoch
+					time_secs = time.mktime(time_struct)
+					
+					# Set modification time
+					os.utime(dst_file, (int(time.time()), time_secs))
+				except:
+					pass
 			
 			gtk.gdk.threads_enter()
 			if transfer_successful:
@@ -1618,16 +1657,26 @@ class PathBar(gtk.Container):
 			self.up_btn.hide()
 			
 	def on_path_btn_clicked(self, widget, path):
-		self.updatePathBtns(path)
-
+		# Deactivate toggle button because the changeDir() may fail. If
+		# changeDir() is successfull updatePathBtns() will activate the toggle
+		# button.
+		handler = widget.get_data('handler_id')
+		widget.handler_block(handler)
+		widget.set_active(False)
+		widget.handler_unblock(handler)
+		
 		if self.fs == 'pvr':
 			path = path.replace('/', '\\')
-			self.guppy.pvr_model.changeDir(path)
+			if self.guppy.changeDir(self.guppy.pvr_model, path) is False:
+				return
 			self.guppy.pvr_path['entry'].set_text(path)
 		else:
-			self.guppy.pc_model.changeDir(path)
+			if self.guppy.changeDir(self.guppy.pc_model, path) is False:
+				return
 			self.guppy.pc_path['entry'].set_text(path)
 
+		self.updatePathBtns(path)
+
 	def on_down_btn_clicked(self, btn):
 		self.queue_resize()
 
