@@ -16,14 +16,14 @@
 ## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 APP_NAME = 'Guppy'
-VERSION = '0.0.92'
+VERSION = '0.0.93'
 DESCRIPTION='Topfield PVR File Downloader'
 AUTHOR = 'Tony Tsui'
 AUTHOR_EMAIL = 'tsui.tony@gmail.com'
 # List of additional contributors. Format: 'Name Email'
 CONTRIBUTORS = [ 'Icons by:', 
                  'Tango Desktop Project http://tango-project.org',
-                 'Peter Appleyard',
+                 'Peter Appleyard peter@whodaman.com.au',
 					  'Translations by:',
                  'Jürgen Bausa',
 ]
